@@ -203,6 +203,104 @@ def example_duplicate_dashboard(
         return None
 
 
+def example_export_dashboard(
+    chronicle: ChronicleClient, dashboard_ids: list
+) -> dict:
+    """Export one or more dashboards.
+
+    Args:
+        chronicle: ChronicleClient instance
+        dashboard_ids: List of dashboard IDs to export
+
+    Returns:
+        Dictionary containing exported dashboard data if successful, None otherwise
+    """
+    print("\n=== Export Dashboard ===")
+
+    try:
+        print(f"\nExporting {len(dashboard_ids)} dashboard(s): {dashboard_ids}")
+        result = chronicle.export_dashboard(dashboard_ids)
+        print("\nDashboard export successful. Result:")
+        print(
+            "- Total dashboards exported: "
+            f'{len(result.get("inlineDestination",{}).get("dashboards", []))}'
+        )
+        # Print a sample of the exported data
+        if result.get("inlineDestination", {}).get("dashboards"):
+            print("\nExported dashboard details:")
+            for i, dashboard in enumerate(
+                result["inlineDestination"]["dashboards"]
+            ):
+                print(f"\nDashboard {i+1}:")
+                print(
+                    f"- Name: {dashboard.get('dashboard').get('name', 'N/A')}"
+                )
+                print(
+                    f"- Display Name: {dashboard.get('dashboard').get('displayName', 'N/A')}"
+                )
+                print(
+                    f"- Description: {dashboard.get('dashboard').get('description', 'N/A')}"
+                )
+                print(
+                    f"- Type: {dashboard.get('dashboard').get('type', 'N/A')}"
+                )
+
+        print("\nFull export response available in the returned result.")
+        return result
+    except Exception as e:  # pylint: disable=broad-exception-caught
+        print(f"Error exporting dashboard: {e}")
+        return None
+
+
+def example_import_dashboard(chronicle: ChronicleClient) -> str:
+    """Import a dashboard from a JSON file.
+
+    Args:
+        chronicle: ChronicleClient instance
+    Returns:
+        New dashboard ID if successful, None otherwise
+    """
+    print("\n=== Import Dashboard ===")
+
+    import_payload = {
+        "dashboard": {
+            "name": "50221a9e-afd7-4f7b-8043-35a925454995",
+            "displayName": "Source Dashboard 8f736a58",
+            "description": "Source dashboard for import test",
+            "definition": {
+                "filters": [
+                    {
+                        "id": "GlobalTimeFilter",
+                        "dataSource": "GLOBAL",
+                        "filterOperatorAndFieldValues": [
+                            {
+                                "filterOperator": "PAST",
+                                "fieldValues": ["1", "DAY"],
+                            }
+                        ],
+                        "displayName": "Global Time Filter",
+                        "isStandardTimeRangeFilter": True,
+                        "isStandardTimeRangeFilterEnabled": True,
+                    }
+                ]
+            },
+            "type": "CUSTOM",
+            "etag": "9bcb466d09e461d19aa890d0f5eb38a5496fa085dc2605954e4457b408acd916",
+            "access": "DASHBOARD_PRIVATE",
+        }
+    }
+
+    try:
+        print("\nImporting dashboard...")
+        result = chronicle.import_dashboard(import_payload)
+        print("Dashboard imported successfully:")
+        print(json.dumps(result, indent=2))
+        if result.get("results"):
+            return result.get("results")[0]["dashboard"].split("/")[-1]
+    except Exception as e:  # pylint: disable=broad-exception-caught
+        print(f"Error importing dashboard: {e}")
+
+
 def example_add_chart(
     chronicle: ChronicleClient, dashboard_id: str
 ) -> Optional[dict]:
@@ -454,6 +552,8 @@ EXAMPLES = {
     "10": example_remove_chart,
     "11": example_execute_dashboard_query,
     "12": example_get_dashboard_query,
+    "13": example_import_dashboard,
+    "14": example_export_dashboard,
 }
 
 
@@ -475,7 +575,7 @@ def main() -> None:
         "--example",
         "-e",
         help=(
-            "Example number to run (1-8). "
+            "Example number to run (1-14). "
             "1: Create Dashboard, "
             "2: Get Dashboard, "
             "3: List Dashboards, "
@@ -486,6 +586,10 @@ def main() -> None:
             "8: Get Chart, "
             "9: Edit Chart, "
             "10: Delete Chart, "
+            "11: Execute Dashboard Query, "
+            "12: Get Dashboard Query, "
+            "13: Import Dashboard, "
+            "14: Export Dashboard, "
             "0: Run All Examples"
         ),
     )
@@ -602,6 +706,31 @@ def main() -> None:
                 return
             example_get_dashboard_query(chronicle, args.query_id)
 
+        elif args.example == "13":
+            # Import Dashboard example
+            new_dashboard_id = example_import_dashboard(chronicle)
+
+            if new_dashboard_id:
+                # Add new dashboard Id for Cleanup
+                dashboard_ids.append(new_dashboard_id)
+
+        elif args.example == "14":
+            # Export Dashboard example
+            if not args.dashboard_id:
+                print(
+                    "Error: At least one dashboard_id is required for this example"
+                )
+                return
+
+            # Convert single dashboard_id to list or split comma-separated values
+            dashboard_ids_to_export = []
+            if "," in args.dashboard_id:
+                dashboard_ids_to_export = args.dashboard_id.split(",")
+            else:
+                dashboard_ids_to_export = [args.dashboard_id]
+
+            example_export_dashboard(chronicle, dashboard_ids_to_export)
+
         elif args.example == "0" or not args.example:
             # Run all examples (create workflow)
             print("=== Running All Examples ===")
@@ -658,6 +787,15 @@ def main() -> None:
 
             example_execute_dashboard_query(chronicle)
 
+            # Import dashboard
+            imported_dashboard_id = example_import_dashboard(chronicle)
+            if imported_dashboard_id:
+                dashboard_ids.append(imported_dashboard_id)
+
+            # Export dashboards
+            if dashboard_ids:
+                example_export_dashboard(chronicle, [imported_dashboard_id])
+
     finally:
         # Clean up all created dashboards
         print("\n=== Cleaning up resources ===")
