@@ -4,7 +4,7 @@ import ipaddress
 import re
 import sys
 from itertools import islice
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Union
 
 from secops.exceptions import APIError, SecOpsError
 
@@ -67,7 +67,8 @@ def create_data_table(
     client: "Any",
     name: str,
     description: str,
-    header: Dict[str, DataTableColumnType],
+    header: Dict[str, Union[DataTableColumnType, str]],
+    column_options: Optional[Dict[str, Dict[str, Any]]] = None,
     rows: Optional[List[List[str]]] = None,
     scopes: Optional[List[str]] = None,
 ) -> Dict[str, Any]:
@@ -77,7 +78,10 @@ def create_data_table(
         client: ChronicleClient instance
         name: The name for the new data table
         description: A user-provided description of the data table
-        header: A dictionary mapping column names to column types
+        header: A dictionary mapping column names to column types or
+                entity proto field mappings
+        column_options: Optional dictionary of column options
+                        syntax: {column_name:{option:value}}
         rows: Optional list of rows for the data table
         scopes: Optional list of scopes for the data table
 
@@ -106,14 +110,28 @@ def create_data_table(
     # Prepare request body
     body_payload = {
         "description": description,
-        "columnInfo": [
-            {"columnIndex": i, "originalColumn": k, "columnType": v.value}
-            for i, (k, v) in enumerate(header.items())
-        ],
+        "columnInfo": [],
     }
+    # Create columnInfo section of request body
+    for i, (column_name, v) in enumerate(header.items()):
+        column = {"columnIndex": i, "originalColumn": column_name}
+
+        # columnType and mappedColumnPath are mutually exclusive
+        if isinstance(v, DataTableColumnType):
+            column["columnType"] = v.value
+        else:  # Assume it is an entity mapping (passed as string).
+            column["mappedColumnPath"] = v
+
+        # Merge additional options into column dictionary
+        if column_options:
+            if column_name in column_options:
+                column = column | column_options[column_name]
+
+        # Finally, add column to list of columns
+        body_payload["columnInfo"].append(column)
 
     if scopes:
-        body_payload["scopes"] = {"dataAccessScopes": scopes}
+        body_payload["scopeInfo"] = {"dataAccessScopes": scopes}
 
     # Create the data table
     response = client.session.post(
@@ -518,10 +536,31 @@ def update_data_table(
     return response.json()
 
 
+def _estimate_row_json_size(row: List[str]) -> int:
+    """Estimate the size of a row when formatted as JSON.
+
+    Args:
+        row: A list of string values representing a row
+
+    Returns:
+        Estimated size in bytes of the row when formatted as JSON
+    """
+    # Basic size for JSON structure: {"data_table_row":{"values":[...]}}
+    base_size = 30
+
+    # Add size for each string value in the row
+    # Account for quotes, commas, and possible escaping
+    for value in row:
+        # String length + quotes + possible escaping (estimate ~10% overhead)
+        base_size += len(value) + 3 + int(len(value) * 0.1)
+
+    return base_size
+
+
 def replace_data_table_rows(
     client: "Any", name: str, rows: List[List[str]]
 ) -> List[Dict[str, Any]]:
-    """Replace all rows in a data table with new rows, chunking if necessary.
+    """Replace all rows in a data table with new rows.
 
     Args:
         client: ChronicleClient instance
@@ -529,81 +568,95 @@ def replace_data_table_rows(
         rows: A list of new rows to replace all existing rows in the table
 
     Returns:
-        List of responses containing the created data table rows
+        List of responses from the API calls
 
     Raises:
-        APIError: If the API request fails
+        APIError: If any API request fails
         SecOpsError: If a row is too large to process
     """
-    responses = []
-    row_iter = iter(rows)
 
-    # Process rows in chunks of up to 1000 rows or 4MB
-    while chunk := list(islice(row_iter, 1000)):
-        current_chunk_size_bytes = sum(
-            sys.getsizeof(".".join(r)) for r in chunk
-        )
-
-        # If chunk is too large, split it
-        while current_chunk_size_bytes > 4000000 and len(chunk) > 1:
-            half_len = len(chunk) // 2
-            if half_len == 0:  # Should not happen if len(chunk) > 1
-                break
-
-            temp_chunk_for_next_iter = chunk[half_len:]
-            chunk = chunk[:half_len]
-            row_iter = iter(temp_chunk_for_next_iter + list(row_iter))
-            current_chunk_size_bytes = sum(
-                sys.getsizeof(".".join(r)) for r in chunk
-            )
-
-        if not chunk:  # If chunk became empty
-            continue
-
-        # If a single row is too large
-        if current_chunk_size_bytes > 4000000 and len(chunk) == 1:
-            raise SecOpsError(
-                "Single row is too large to process "
-                f"(>{current_chunk_size_bytes} bytes): {chunk[0][:100]}..."
-            )
-
-        responses.append(_replace_data_table_rows(client, name, chunk))
-
-    return responses
-
-
-def _replace_data_table_rows(
-    client: "Any", name: str, rows: List[List[str]]
-) -> Dict[str, Any]:
-    """Replace all rows in a data table with new rows in a single request.
-
-    Args:
-        client: ChronicleClient instance
-        name: The name of the data table
-        rows: New data table rows to replace all existing rows.
-              A maximum of 1000 rows can be replaced in a single request.
-              Total size of the rows should be less than 4MB.
-
-    Returns:
-        Dictionary containing the data table rows that replaced existing
-        data table rows
-
-    Raises:
-        APIError: If the API request fails
-    """
     url = (
         f"{client.base_url}/{client.instance_id}/dataTables/{name}"
         "/dataTableRows:bulkReplace"
     )
-    response = client.session.post(
-        url,
-        json={"requests": [{"data_table_row": {"values": x}} for x in rows]},
-    )
 
-    if response.status_code != 200:
-        raise APIError(
-            f"Failed to replace data table rows for '{name}': "
-            f"{response.status_code} {response.text}"
+    # Check for empty input
+    if not rows:
+        return []
+
+    row_sizes = []
+
+    # Validate each row isn't too large before processing
+    for i, row in enumerate(rows):
+        # Validate row structure to prevent IndexError
+        if not isinstance(row, list):
+            raise SecOpsError(
+                f"Invalid row format at index {i}: "
+                f"expected list but got {type(row)}"
+            )
+
+        row_size = _estimate_row_json_size(row)
+        row_sizes.append(row_size)  # Store calculated size
+
+        if row_size > 4000000:
+            row_preview = repr(row[:50])
+            raise SecOpsError(
+                f"Single row is too large to process (>{row_size} bytes): "
+                f"{row_preview}..."
+            )
+
+    all_responses = []
+
+    # Use bulkReplace for the first batch (up to 1000 rows)
+    # This replaces all existing rows in the table
+    first_batch_size = min(1000, len(rows))
+    first_batch = rows[:first_batch_size]
+    first_batch_sizes = row_sizes[:first_batch_size]
+
+    print(f"Replacing all existing rows with first {first_batch_size} rows")
+
+    first_api_batch = []
+    first_api_batch_size = 0
+    first_batch_max_size = 4000000  # 4MB in bytes
+
+    # First, determine how many rows we can include in the first API call
+    # (max 4MB)
+    for i, row in enumerate(first_batch):
+        # If adding this row would exceed 4MB,
+        # stop adding rows to first API call
+        if first_api_batch_size + first_batch_sizes[i] > first_batch_max_size:
+            break
+
+        first_api_batch.append(row)
+        first_api_batch_size += first_batch_sizes[i]
+
+    if first_api_batch:
+        replace_requests = [
+            {"data_table_row": {"values": r}} for r in first_api_batch
+        ]
+
+        response = client.session.post(
+            url,
+            json={"requests": replace_requests},
         )
 
-    return response.json()
+        if response.status_code != 200:
+            raise APIError(
+                f"Failed to replace data table rows for '{name}': "
+                f"{response.status_code} {response.text}"
+            )
+
+        all_responses.append(response.json())
+
+    # Handle any remaining rows from the first 1000 using bulkCreate
+    remaining_first_batch = first_batch[len(first_api_batch) :]
+
+    # Add remaining rows using bulkCreate (if any)
+    if remaining_first_batch or len(rows) > 1000:
+        print(f"Adding remaining {len(rows) - len(first_api_batch)} rows")
+
+        remaining_rows = remaining_first_batch + rows[1000:]
+        create_responses = create_data_table_rows(client, name, remaining_rows)
+        all_responses.extend(create_responses)
+
+    return all_responses
