@@ -688,6 +688,64 @@ def test_chronicle_rule_detections():
         pytest.fail(f"API Error during rule detections test: {str(e)}")
 
 
+@pytest.mark.integration
+def test_chronicle_rule_detections_with_time_range():
+    """Test Chronicle rule detections functionality with time range parameters."""
+    client = SecOpsClient()
+    chronicle = client.chronicle(**CHRONICLE_CONFIG)
+
+    # Use the specific rule ID provided
+    rule_id = "ru_b2caeac4-c3bd-4b61-9007-bd1e481eff85"
+    
+    # Set up time range for testing
+    end_time = datetime.now(timezone.utc)
+    start_time = end_time - timedelta(days=7)  # Look back 7 days
+
+    try:
+        # Test with time range parameters
+        detections = chronicle.list_detections(
+            rule_id,
+            start_time=start_time,
+            end_time=end_time
+        )
+        
+        assert isinstance(detections, dict)
+        print(f"Successfully retrieved detections with time range for rule {rule_id}")
+        
+    except APIError as e:
+        pytest.fail(f"API Error during rule detections time range test: {str(e)}")
+
+
+@pytest.mark.integration
+def test_chronicle_rule_detections_with_list_basis():
+    """Test Chronicle rule detections functionality with list_basis parameter."""
+    client = SecOpsClient()
+    chronicle = client.chronicle(**CHRONICLE_CONFIG)
+
+    # Use the specific rule ID provided
+    rule_id = "ru_b2caeac4-c3bd-4b61-9007-bd1e481eff85"
+    
+    # Set up time range for testing
+    end_time = datetime.now(timezone.utc)
+    start_time = end_time - timedelta(days=7)  # Look back 7 days
+
+    try:
+        # Test with different list_basis values
+        for list_basis in ["LIST_BASIS_UNSPECIFIED", "DETECTION_TIME", "CREATED_TIME"]:
+            detections = chronicle.list_detections(
+                rule_id,
+                start_time=start_time,
+                end_time=end_time,
+                list_basis=list_basis
+            )
+            
+            assert isinstance(detections, dict)
+            print(f"Successfully retrieved detections with list_basis={list_basis}")
+        
+    except APIError as e:
+        pytest.fail(f"API Error during rule detections list_basis test: {str(e)}")
+
+
 @pytest.mark.integration
 def test_chronicle_rule_validation():
     """Test Chronicle rule validation functionality with real API."""
@@ -736,6 +794,67 @@ rule test_rule {
         pytest.fail(f"API Error during rule validation test: {str(e)}")
 
 
+@pytest.mark.integration
+def test_chronicle_fetch_udm_search_view():
+    """Test fetch_udm_search_view functionality with real API.
+
+    This test verifies that the fetch_udm_search_view function can successfully
+    connect to the Chronicle API and retrieve UDM search results.
+    """
+    client = SecOpsClient(service_account_info=SERVICE_ACCOUNT_JSON)
+    chronicle = client.chronicle(**CHRONICLE_CONFIG)
+
+    # Use a reasonable time window (1 hour)
+    end_time = datetime.now(timezone.utc)
+    start_time = end_time - timedelta(hours=1)
+
+    # Use a common event type that should exist in most environments
+    query = 'metadata.event_type = "NETWORK_CONNECTION"'
+
+    try:
+        print("\nStarting fetch_udm_search_view integration test...")
+        print(f"Time window: {start_time.isoformat()} to {end_time.isoformat()}")
+        print(f"Query: {query}")
+
+        # Execute the function being tested
+        result = chronicle.fetch_udm_search_view(
+            query=query,
+            start_time=start_time,
+            end_time=end_time,
+            max_events=5,  # Limit results for quicker testing
+            case_insensitive=True,
+        )
+
+        # Basic validation of the response
+        assert isinstance(result, list), "Result should be a list"
+        assert len(result) > 0, "Result list should not be empty"
+        
+        if result[0].get("complete") is True:
+            print("\nSearch completed successfully")
+            
+            # Check if events were found
+            events = result[0].get("events", {}).get("events", [])
+            print(f"Found {len(events)} events")
+            
+            if events:
+                # Validate structure of the first event
+                event = events[0]
+                assert "id" in event.get("event",{}).get("metadata"), "Event should have an ID"
+                assert "metadata" in event.get("event",{}), "Event should have metadata"
+            
+            else:
+                pytest.skip("No events found in the specified time range")
+        else:
+            pytest.skip("Search did not complete successfully")
+
+    except APIError as e:
+        print(f"\nAPI Error: {str(e)}")
+        # Skip the test rather than fail if service unavailable or permissions issue
+        if any(msg in str(e).lower() for msg in ["permission", "not authorized", "unavailable"]):
+            pytest.skip(f"Test skipped due to API error: {str(e)}")
+        raise
+
+
 @pytest.mark.integration
 def test_chronicle_nl_search():
     """Test Chronicle natural language search functionality with real API."""
@@ -794,90 +913,6 @@ def test_chronicle_nl_search():
             raise
 
 
-@pytest.mark.integration
-def test_chronicle_data_export():
-    """Test Chronicle data export functionality with real API."""
-    client = SecOpsClient(service_account_info=SERVICE_ACCOUNT_JSON)
-    chronicle = client.chronicle(**CHRONICLE_CONFIG)
-
-    # Set up time range for testing
-    end_time = datetime.now(timezone.utc)
-    start_time = end_time - timedelta(days=14)  # Look back 1 day
-
-    try:
-        # First, fetch available log types
-        log_types_result = chronicle.fetch_available_log_types(
-            start_time=start_time,
-            end_time=end_time,
-            page_size=10,  # Limit to 10 for testing
-        )
-
-        print(
-            f"\nFound {len(log_types_result['available_log_types'])} available log types for export"
-        )
-
-        # If no log types available, skip the test
-        if not log_types_result["available_log_types"]:
-            pytest.skip("No log types available for export in the specified time range")
-
-        # Show some of the available log types
-        for log_type in log_types_result["available_log_types"][:3]:  # Show first 3
-            print(f"  {log_type.display_name} ({log_type.log_type.split('/')[-1]})")
-            print(f"  Available from {log_type.start_time} to {log_type.end_time}")
-
-        # For the actual export test, we'll create an export but not wait for completion
-        # Choose a log type that's likely to be present
-        if log_types_result["available_log_types"]:
-            selected_log_type = log_types_result["available_log_types"][
-                0
-            ].log_type.split("/")[-1]
-
-            # Create a data export (this might fail if the GCS bucket isn't properly set up)
-            try:
-                # This part would require a valid GCS bucket to work properly
-                # We'll make the request but catch and report errors without failing the test
-                bucket_name = "dk-test-export-bucket"
-
-                export = chronicle.create_data_export(
-                    gcs_bucket=f"projects/{CHRONICLE_CONFIG['project_id']}/buckets/{bucket_name}",
-                    start_time=start_time,
-                    end_time=end_time,
-                    log_type=selected_log_type,
-                )
-
-                print(f"\nCreated data export for log type: {selected_log_type}")
-                print(f"Export ID: {export['name'].split('/')[-1]}")
-                print(f"Status: {export['data_export_status']['stage']}")
-
-                # Test the get_data_export function
-                export_id = export["name"].split("/")[-1]
-                export_status = chronicle.get_data_export(export_id)
-                print(
-                    f"Retrieved export status: {export_status['data_export_status']['stage']}"
-                )
-
-                # Cancel the export
-                cancelled = chronicle.cancel_data_export(export_id)
-                print(
-                    f"Cancelled export status: {cancelled['data_export_status']['stage']}"
-                )
-
-            except APIError as e:
-                # Don't fail the test if export creation fails due to permissions
-                # (GCS bucket access, etc.)
-                print(f"\nData export creation failed: {str(e)}")
-                print(
-                    "This is expected if GCS bucket isn't configured or permissions are missing."
-                )
-
-    except APIError as e:
-        print(f"\nAPI Error details: {str(e)}")  # Debug print
-        # If we get "not found" or permission errors, skip rather than fail
-        if "permission" in str(e).lower() or "not found" in str(e).lower():
-            pytest.skip(f"Skipping due to permission issues: {str(e)}")
-        raise
-
-
 @pytest.mark.integration
 def test_chronicle_batch_log_ingestion():
     """Test batch log ingestion with real API."""
@@ -1271,9 +1306,11 @@ def test_chronicle_data_tables():
                 "ip_address": DataTableColumnType.STRING,
                 "description": DataTableColumnType.STRING,
             },
+            column_options={"ip_address": {"repeatedValues": True}},
             rows=[
                 ["host1.example.com", "192.168.1.10", "Primary server"],
                 ["host2.example.com", "192.168.1.11", "Backup server"],
+                ["host3.example.com", "192.168.1.10", "Proxy server"],
             ],
         )
 
@@ -1289,7 +1326,7 @@ def test_chronicle_data_tables():
         # List rows
         rows = chronicle.list_data_table_rows(dt_name)
         print(f"Found {len(rows)} rows in data table")
-        assert len(rows) == 2  # We added 2 rows during creation
+        assert len(rows) == 3  # We added 3 rows during creation
 
         # Store row IDs for deletion testing
         row_ids = [
@@ -1304,7 +1341,7 @@ def test_chronicle_data_tables():
 
             # Check rows after deletion
             updated_rows = chronicle.list_data_table_rows(dt_name)
-            assert len(updated_rows) == 1  # Should be one less than before
+            assert len(updated_rows) == 2  # Should be one less than before
 
         # Add more rows
         new_rows = [
@@ -1316,7 +1353,7 @@ def test_chronicle_data_tables():
 
         # Check rows after addition
         final_rows = chronicle.list_data_table_rows(dt_name)
-        assert len(final_rows) == 3  # 1 remaining + 2 new ones
+        assert len(final_rows) == 4  # 2 remaining + 2 new ones
 
     except Exception as e:
         print(f"Error during data table test: {e}")
