@@ -168,6 +168,26 @@ def test_list_rules(chronicle_client, mock_response):
         assert len(result["rules"]) == 2
 
 
+def test_list_rules_empty(chronicle_client, mock_response):
+    """Test list_rules function with no rules"""
+    # Arrange
+    mock_response.json.return_value = {}
+
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_response
+    ) as mock_get:
+        # Act
+        result = list_rules(chronicle_client)
+
+        # Assert
+        mock_get.assert_called_once_with(
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules",
+            params={"pageSize": 1000, "view": "FULL"},
+        )
+        assert result == {"rules": []}
+        assert len(result["rules"]) == 0
+
+
 def test_list_rules_pagination(chronicle_client):
     """Test list_rules function with pagination."""
 
@@ -369,9 +389,9 @@ def test_enable_rule_error(chronicle_client, mock_error_response):
     ):
         # Act & Assert
         with pytest.raises(APIError) as exc_info:
-            enable_rule(chronicle_client, rule_id)
+            enable_rule(chronicle_client, rule_id, True)
 
-        assert "Failed to enable rule" in str(exc_info.value)
+        assert "Failed to update rule deployment" in str(exc_info.value)
 
 
 def test_search_rules(chronicle_client, mock_response):
@@ -537,3 +557,160 @@ def test_run_rule_test_handles_exceptions(chronicle_client):
             )
 
         assert "Error testing rule" in str(exc_info.value)
+
+
+def test_get_rule_deployment(chronicle_client, mock_response):
+    """Test get_rule_deployment function."""
+    rule_id = "ru_12345"
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_response
+    ) as mock_get:
+        from secops.chronicle.rule import get_rule_deployment
+
+        result = get_rule_deployment(chronicle_client, rule_id)
+
+        mock_get.assert_called_once_with(
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}/deployment"
+        )
+        assert result == mock_response.json.return_value
+
+
+def test_get_rule_deployment_error(chronicle_client, mock_error_response):
+    """Test get_rule_deployment function with error response."""
+    rule_id = "ru_12345"
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_error_response
+    ):
+        from secops.chronicle.rule import get_rule_deployment
+
+        with pytest.raises(APIError) as exc_info:
+            get_rule_deployment(chronicle_client, rule_id)
+
+        assert "Failed to get rule deployment" in str(exc_info.value)
+
+
+def test_list_rule_deployments_single_page(chronicle_client, mock_response):
+    """Test list_rule_deployments with a single page."""
+    mock_response.json.return_value = {
+        "ruleDeployments": [{"name": "dep1"}, {"name": "dep2"}]
+    }
+
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_response
+    ) as mock_get:
+        from secops.chronicle.rule import list_rule_deployments
+
+        result = list_rule_deployments(chronicle_client)
+
+        mock_get.assert_called_once_with(
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/-/deployments",
+            params={},
+        )
+        assert result == {
+            "ruleDeployments": [{"name": "dep1"}, {"name": "dep2"}]
+        }
+
+
+def test_list_rule_deployments_pagination(chronicle_client):
+    """Test list_rule_deployments function with pagination."""
+
+    def get_mock_response(url, **kwargs):
+        params = kwargs.get("params", {})
+        if "pageToken" not in params:
+            # First page response
+            response = Mock()
+            response.status_code = 200
+            response.json.return_value = {
+                "ruleDeployments": [{"name": "dep1"}],
+                "nextPageToken": "page2token",
+            }
+            return response
+        elif params["pageToken"] == "page2token":
+            # Second page response
+            response = Mock()
+            response.status_code = 200
+            response.json.return_value = {
+                "ruleDeployments": [{"name": "dep2"}],
+            }
+            return response
+        else:
+            raise ValueError(f"Unexpected pageToken: {params.get('pageToken')}")
+
+    with patch.object(chronicle_client.session, "get") as mock_get:
+        mock_get.side_effect = get_mock_response
+
+        from secops.chronicle.rule import list_rule_deployments
+
+        result = list_rule_deployments(chronicle_client)
+
+        assert mock_get.call_count == 2
+        assert [d["name"] for d in result["ruleDeployments"]] == [
+            "dep1",
+            "dep2",
+        ]
+
+
+def test_list_rule_deployments_error(chronicle_client, mock_error_response):
+    """Test list_rule_deployments function with error response."""
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_error_response
+    ):
+        from secops.chronicle.rule import list_rule_deployments
+
+        with pytest.raises(APIError) as exc_info:
+            list_rule_deployments(chronicle_client)
+
+        assert "Failed to list rule deployments" in str(exc_info.value)
+
+
+def test_list_rule_deployments_empty(chronicle_client, mock_response):
+    """Test list_rule_deployments function with no rule deployments."""
+    # Arrange
+    mock_response.json.return_value = {}
+
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_response
+    ) as mock_get:
+        from secops.chronicle.rule import list_rule_deployments
+
+        # Act
+        result = list_rule_deployments(chronicle_client)
+
+        # Assert
+        mock_get.assert_called_once_with(
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/-/deployments",
+            params={},
+        )
+        assert result == {"ruleDeployments": []}
+        assert len(result["ruleDeployments"]) == 0
+
+
+def test_list_rule_deployments_with_filter(chronicle_client, mock_response):
+    """Test list_rule_deployments function with filter parameter."""
+    # Arrange
+    mock_response.json.return_value = {
+        "ruleDeployments": [{"name": "filtered_deployment", "enabled": True}]
+    }
+
+    with patch.object(
+        chronicle_client.session, "get", return_value=mock_response
+    ) as mock_get:
+        from secops.chronicle.rule import list_rule_deployments
+
+        # Act
+        filter_query = "enabled=true"
+        result = list_rule_deployments(
+            chronicle_client, filter_query=filter_query
+        )
+
+        # Assert
+        mock_get.assert_called_once_with(
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/-/deployments",
+            params={"filter": filter_query},
+        )
+        assert result == {
+            "ruleDeployments": [
+                {"name": "filtered_deployment", "enabled": True}
+            ]
+        }
+        assert len(result["ruleDeployments"]) == 1
